<?php
/*  Copyright 2006  ADY ROMANTIKA  (email : ady@romantika.name)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
Plugin Name: Blogger RSS Import
Plugin URI: http://www.romantika.name/v2/2007/01/31/import-new-blogger-to-wordpress/
Description: Previously known as New Blogger Import. This plugin imports Blogger posts and comments into WordPress using RSS feeds
Author: Ady Romantika
Version: 1.1
Author URI: http://www.romantika.name/v2/

Changelog:
=========
2006-12-22 -	1. added ability to add commentor URL and email since these data can't be obtained from RSS.
				2. added extra checks to terminate script if something fails. No use wasting user time to wait for a broken result.
2006-01-01 -	1. Bugfix: if comment author's name contains an apostrophe it failed. Not anymore.
2007-01-10 -	1. added checking for available retrieving mechanism available on the server. Notify and die if none supported.
				2. added code utilizing curl to download file - suggested by Charles Wesley
				3. added support for incremental import to cope with speed problems
2007-01-14 -	1. Bugfix: import only works on second try
2007-01-31 -	1. As a plugin
				2. Removed session, now use POST
				3. Haloscan friendly import - requested by John Halton
				4. Matching of authors
				5. Matching of categories (labels)
				6. Ability to choose staged or all-at-once import
				7. Maintain permalinks - suggested by Stephen Newton
2007-02-23 -	1. Bugfix: Problems with users having dash in the blogger ID
				2. Bugfix: Strip .htm as well, instead of only .html
2007-02-26 -	1. Changed method to get comments - now use real RSS instead of ATOM. Eliminates the maximum 25 comments imported
				2. Added fix for WordPress MU as contributed by Sander van Dragt
				3. Enables the ability not to import comments - speeds up users using Haloscan, or simply don't want any comments imported
				4. Updated to version 1.1
				5. Added ability to skip any cycle, even the first one
*/



// global variables
$currentElements = array();
$itemCount = 0;
$postsArray = array();
$currentCElements = array();
$itemCCount = 0;
$commentsArray = array();
$mydata = array();

// WordPress MU < 2.2 fix (Contributed by Sander van Dragt <sander.vandragt@gmail.com>)
$tableposts = $wpdb->posts;
$tableusers = $wpdb->users;
$tablecomments = $wpdb->comments;
$tablecategories = $wpdb->categories;

/***********************************************************************
/ POSTS
/**********************************************************************/
function readXML($xmlFile)
{
	global $mydata;
	$xmlParser = xml_parser_create();

	xml_parser_set_option($xmlParser, XML_OPTION_CASE_FOLDING, false);
	xml_set_element_handler($xmlParser, startElement, endElement);
	xml_set_character_data_handler($xmlParser, characterData);

	switch($mydata['mechanism'])
	{
		case 'curl':
			$filedata = ara_blogger_import_getCURL($xmlFile);
	        	xml_parse($xmlParser, $filedata, true);
		break;
		case 'fopen':
			$fp = fopen($xmlFile, "r") or die('Unable to extract post from '.$xmlFile);
			while($data = fread($fp, 8192))
			{
				xml_parse($xmlParser, $data, feof($fp));
			}
		break;
	}
	xml_parser_free($xmlParser);
}

// Sets the current XML element, and pushes itself onto the element hierarchy
function startElement($parser, $name, $attrs)
{
	global $currentElements, $itemCount;
	array_push($currentElements, $name);
	if($name == "item"){$itemCount += 1;}
}

// Prints XML data; finds highlights and links
function characterData($parser, $data)
{
	global $currentElements, $postsArray, $itemCount;

	$currentCount = count($currentElements);
	$parentElement = $currentElements[$currentCount-2];
	$thisElement = $currentElements[$currentCount-1];

	if($parentElement == "item")
	{
		if($thisElement == 'category')
			 $postsArray[$itemCount-1][$thisElement] .= $data . '#cat#';
		else
			$postsArray[$itemCount-1][$thisElement] .= $data;
	}
}

// If the XML element has ended, it is poped off the hierarchy
function endElement($parser, $name)
{
	global $currentElements;

	$currentCount = count($currentElements);
	if($currentElements[$currentCount-1] == $name)
	{
		array_pop($currentElements);
	}
}

/***********************************************************************
/ COMMENTS
/**********************************************************************/
// Reads XML file into formatted html
function readCXML($xmlFile)
{
	global $mydata;
	$xmlParser = xml_parser_create();

	xml_parser_set_option($xmlParser, XML_OPTION_CASE_FOLDING, false);
	xml_set_element_handler($xmlParser, startCElement, endCElement);
	xml_set_character_data_handler($xmlParser, characterCData);

	switch($mydata['mechanism'])
	{
		case 'curl':
	        	$filedata = ara_blogger_import_getCURL($xmlFile);
	        	xml_parse($xmlParser, $filedata);
		break;
		case 'fopen':
			$fp = fopen($xmlFile, "r") or die('<br/><font color="red">ERROR: ['.mysql_error().']</font> Unable to extract comments from '.$xmlFile);
			while($data = fread($fp, 8192))
			{
				xml_parse($xmlParser, $data, feof($fp));
			}
		break;
	}
	xml_parser_free($xmlParser);
}

// Sets the current XML element, and pushes itself onto the element hierarchy
function startCElement($parser, $name, $attrs)
{
	global $currentCElements, $itemCCount;
	array_push($currentCElements, $name);
	if($name == "item"){$itemCCount += 1;}
}

// Prints XML data; finds highlights and links
function characterCData($parser, $data)
{
	global $currentCElements, $commentsArray, $itemCCount;

	$currentCount = count($currentCElements);
	$parentElement = $currentCElements[$currentCount-2];
	$thisElement = $currentCElements[$currentCount-1];

	if($parentElement == "item")
	{
		$commentsArray[$itemCCount-1][$thisElement] .= $data;
	}
}

// If the XML element has ended, it is poped off the hierarchy
function endCElement($parser, $name)
{
	global $currentCElements;

	$currentCount = count($currentCElements);
	if($currentCElements[$currentCount-1] == $name)
	{
		array_pop($currentCElements);
	}
}

function ara_blogger_import_getCURL($url)
{
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, $url);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	curl_setopt($ch, CURLOPT_HEADER, 0);
	$filedata = curl_exec($ch);
	curl_close($ch);
	return $filedata;
}

function ara_blogger_import_showthanks()
{
	global $mydata;
	echo '<p>Thanks for using this plugin.</p>';
	echo '<p><strong>Some points worth noting:</strong></p>';
	echo '<ul>';
	if(strlen($mydata['matchauthors']) > 0) echo '<li>New users are created as subscribers and empty passwords.</li>';
	echo '</ul>';
	echo '<p>If you think of anything else that can be listed here, do let me know.</p>';
}

function ara_blogger_import_process($skip = false)
{
	global $wpdb, $tableposts, $tablecomments, $mydata, $postsArray, $currentElements, $itemCount; //, $commentsArray, $currentCElements, $itemCCount;

	echo '<div class=wrap>';
	echo '<h2>Blogger RSS Import v1.1</h2>';
	print '<p>Blogger ID: ['.$mydata['bloggerId'].']';
	print '<br/>Number of posts: ['.$mydata['totalPosts'].']';
	if(strlen(trim($mydata['allatonce'])) == 0)
	{
		print '<br/>Number of cycles: ['.$mydata['numLoop'].']';
		print '<br/>Current cycle: ['.($mydata['currLoop']).']';
		print '<br/>Number of cycles to go: ['.($mydata['currLoop']-1).']';
	}
	print '<br/>Download mechanism: ['.$mydata['mechanism'].']';
	print '<br/>Haloscan friendly: ['.(strlen($mydata['importhalo']) > 0 ? $mydata['importhalo'] : 'false').']';
	print '<br/>Do not import comments: ['.(strlen($mydata['nocomments']) > 0 ? $mydata['nocomments'] : 'false').']';
	print '<br/>Maintain permalinks: ['.(strlen($mydata['maintainperma']) > 0 ? $mydata['maintainperma'] : 'false').']';
	print '<br/>Import all entries at once: ['.(strlen($mydata['allatonce']) > 0 ? $mydata['allatonce'] : 'false').']';
	print '<br/>Match categories: ['.(strlen($mydata['matchcats']) > 0 ? $mydata['matchcats'] : 'false').']';
	print '<br/>Match authors: ['.(strlen($mydata['matchauthors']) > 0 ? $mydata['matchauthors'] : 'false').']';
	print '<br/>';

	if(!$skip) :

		if(strlen($mydata['allatonce']) > 0)
		{
			for($l=$mydata['currLoop'];$l>0;$l--)
			{
				readXML($mydata['post_url'].($mydata['currLoop'] * $mydata['numposts'] - ($mydata['numposts'] - 1)),$mydata['mechanism']);
				$mydata['currLoop']--;
			}
			$mydata['currLoop'] = 1;
		}
		else
			readXML($mydata['post_url'].($mydata['currLoop'] * $mydata['numposts'] - ($mydata['numposts'] - 1)),$mydata['mechanism']);

		$postsArray = array_reverse($postsArray);

		// Now that we have the data
		for($x=0;$x<$itemCount;$x++)
		{
			global  $commentsArray, $currentCElements, $itemCCount;
			unset($to_insert);
			$commentsArray = array();
			$currentCElements = array();
			$itemCCount = 0;
			$to_insert = array();

			// Get blogpost ID - useful to retrieve comments later
			$postsArray[$x]['blogpost-id'] = substr($postsArray[$x]['guid'],strpos($postsArray[$x]['guid'],'.post-')+6);
			// Debugging display
			if($mydata['currLoop'] == $mydata['numLoop'] || strlen($mydata['allatonce']) > 0)
				print "<br/>Post ".($mydata['totalPosts'] - $x)." read from XML: ".$postsArray[$x]['title']; //." - ".$postsArray[$x]['blogpost-id'];
			else
				print "<br/>Post ".($mydata['numposts'] * $mydata['currLoop'] - $x)." read from XML: ".$postsArray[$x]['title']; //." - ".$postsArray[$x]['blogpost-id'];

			// Cleanup of javascript onblur thingies from images
			$postsArray[$x]['description'] = preg_replace('/\sonblur=\"(\w*\{*\}*\s*\.*\(*\)*\;*)*\"/','',$postsArray[$x]['description']);

			// Wed, 13 Dec 2006 15:27:00 +0000
			$datetime = strtotime($postsArray[$x]['pubDate']);
			$postDate = date("Y-m-d H:i:s",$datetime);
			$postDateGMT = gmdate("Y-m-d H:i:s",$datetime);

			if(strlen($mydata['matchcats']) > 0)
			{
				unset($cat_IDs);
				$exploded_cats = explode('#cat#', $postsArray[$x]['category']);
				for($c=0;$c<count($exploded_cats)-1;$c++)
				{
					$result = category_exists($exploded_cats[$c]);
					if(empty($result))
					{
						$cat_IDs[] = wp_create_category(ucwords($exploded_cats[$c]));
						echo '<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;New category <strong>'.ucwords($exploded_cats[$c]).'</strong> found';
					}
					else $cat_IDs[] = $result;
				}
			}

			if(strlen($mydata['matchauthors']) > 0)
			{
				$uid = username_exists($postsArray[$x]['author']);
				if(empty($uid) or is_null($uid))
				{
					$user_insert['user_login'] = $postsArray[$x]['author'];
					$user_insert['user_nicename'] = $postsArray[$x]['author'];
					$user_insert['display_name'] = $postsArray[$x]['author'];
					$uid = wp_insert_user($user_insert);
					echo '<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;New user <strong>'.$postsArray[$x]['author'].'</strong> found';
				}
			}

			$to_insert['post_category'] = (strlen($mydata['matchcats']) > 0 && count($cat_IDs) > 0 ? $cat_IDs : $mydata['category']);
			$to_insert['post_author'] = (strlen($mydata['matchauthors']) > 0 ? $uid : $mydata['user']);
			$to_insert['post_date'] = $postDate;
			$to_insert['post_date_gmt'] = $postDateGMT;
			if(strlen($mydata['importhalo']) > 0)
				$to_insert['post_content'] = '<!--' . $postsArray[$x]['blogpost-id'] . '-->' .
					str_replace(array('<br>','<BR>','<br/>','<BR/>','<br />','<BR />'), "\n", addslashes($postsArray[$x]['description']));
			else
				$to_insert['post_content'] = str_replace(array('<br>','<BR>','<br/>','<BR/>','<br />','<BR />'), "\n", addslashes($postsArray[$x]['description']));
			$to_insert['post_title'] = addslashes($postsArray[$x]['title']);
			$to_insert['post_status'] = 'publish';
			if(strlen($mydata['maintainperma']) > 0)
			{
				// Get post name from blogger
				$link = preg_replace('/http:\/\/\w+(-|_*\w*)*.\w+.\w+\/\d+\/\d+\//','',$postsArray[$x]['link']);
				$link = str_replace('.html','',$link);
				$link = str_replace('.htm','',$link);
				$to_insert['post_name'] = $link;
				echo '<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Permalink captured: <a target="_blank" href="'.$postsArray[$x]['link'].'">'.$link.'</a>';
			}
			else
				$to_insert['post_name'] = sanitize_title_with_dashes($postsArray[$x]['title']);
			$Post_ID = wp_insert_post($to_insert);

			if(strlen($mydata['nocomments']) == 0)
			{
				// Look for the number of comments in this post
				$number_comments = ara_blogger_import_getnumcomments($postsArray[$x]['blogpost-id']);

				// Setting number of loops, by default take 100 comments at a time (hopefully won't be a problem)
				$numCLoop = ceil($number_comments/100);
				// Extract comments based on post ID
				for($cc=0;$cc<$numCLoop;$cc++){
					readCXML(str_replace('XXXXXXXXXXXX',$postsArray[$x]['blogpost-id'],$mydata['comment_url']).(($cc * 100) + 1),$mydata['mechanism']);
				}

				for($y=0;$y<$itemCCount;$y++)
				{
					$datetime = strtotime($commentsArray[$x]['pubDate']);
					$commentDate = date("Y-m-d H:i:s",$datetime);
					$commentDateGMT = gmdate("Y-m-d H:i:s",$datetime);

					$comment_insert['comment_post_ID'] = $Post_ID;
					$comment_insert['comment_author'] = addslashes($commentsArray[$y]['author']);
					$comment_insert['comment_author_email'] = '';
					$comment_insert['comment_author_url'] = (isset($commentsArray[$y]['uri']) ? $commentsArray[$y]['uri'] : '');
					$comment_insert['comment_date'] = $commentDate;
					$comment_insert['comment_date_gmt'] = $commentDateGMT;
					$comment_insert['comment_content'] = addslashes($commentsArray[$y]['description']);
					$comment_insert['comment_approved'] = '1';
					$comment_insert['user_id'] = 0;
					wp_insert_comment($comment_insert);
				}
				echo "<br/>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; $y comments captured along...";
			}
		}

	else:
		echo '<p><font color="red">Cycle skipped</font></p>';

	endif;

	if($mydata['currLoop'] == 1)
	{
		echo '<h2>Blogger RSS Import v1.1</h2>';
		echo '<p>You\'re done!</p>';

		ara_blogger_import_showthanks();

		echo '<p>Now if you want... you can map these comment authors to their website and emails, below.</p>';

		// Now look for comments author if user wants to
		$comment_authors = $wpdb->get_results("SELECT comment_author,count(comment_author) AS count,comment_author_email,comment_author_url FROM $tablecomments GROUP BY comment_author ORDER BY count DESC");
		$numrows = 0;
		echo '<form method="POST" action="">';
		echo '<table border="1">';
		echo '<tr><td>Author</td><td>No of comments</td><td>Email</td><td>Website (please include http://)</td></tr>';
		foreach ($comment_authors as $author)
		{
			echo '<tr>';
			echo '<td>'.$author->comment_author.'</td>';
			echo '<td>'.$author->count.'</td>';
			echo '<td><input type="text" name="email'.$numrows.'" value="'.$author->comment_author_email.'"></td>';
			echo '<td><input type="text" name="website'.$numrows.'" value="'.$author->comment_author_url.'"><input type="hidden" name="author'.$numrows.'" value="'.$author->comment_author.'"></td>';
			echo '</tr>';
			$numrows++;
		}
		echo '</table>';
		echo '<input type="hidden" name="commentcount" value="'.$numrows.'">';
		echo '<input type="submit" value="Save">';
		echo '</form>';
	}
	else
	{
		echo '<p>You are <strong>not</strong> done, there are more posts to be imported.</p>';
		echo '<form method="POST" action="">';
		echo '<input type="hidden" name="blogger" value="'.$mydata['blogger'].'">';
		echo '<input type="hidden" name="bloggerId" value="'.$mydata['bloggerId'].'">';
		echo '<input type="hidden" name="numposts" value="'.$mydata['numposts'].'">';
		echo '<input type="hidden" name="totalPosts" value="'.$mydata['totalPosts'].'">';
		echo '<input type="hidden" name="numLoop" value="'.$mydata['numLoop'].'">';
		echo '<input type="hidden" name="currLoop" value="'.$mydata['currLoop'].'">';
		echo '<input type="hidden" name="basic_url" value="'.$mydata['basic_url'].'">'; // this might not be needed anymore
		echo '<input type="hidden" name="post_url" value="'.$mydata['post_url'].'">';
		echo '<input type="hidden" name="nocomments" value="'.$mydata['nocomments'].'">';
		echo '<input type="hidden" name="comment_url" value="'.$mydata['comment_url'].'">';
		echo '<input type="hidden" name="mechanism" value="'.$mydata['mechanism'].'">';
		echo '<input type="hidden" name="importhalo" value="'.$mydata['importhalo'].'">';
		echo '<input type="hidden" name="maintainperma" value="'.$mydata['maintainperma'].'">';
		echo '<input type="hidden" name="matchcats" value="'.$mydata['matchcats'].'">';
		echo '<input type="hidden" name="matchauthors" value="'.$mydata['matchauthors'].'">';
		echo '<input type="submit" name="next" value="Next &gt;&gt;">&nbsp;<input type="submit" name="skip" value="Skip Next Cycle &gt;&gt;">';
		echo '</form>';
	}
}

function ara_blogger_import_getmetadata($post)
{
	global $mydata;
	switch($mydata['mechanism'])
	{
		case 'curl':
			$filedata = ara_blogger_import_getCURL($mydata['basic_url']);
		break;
		case 'fopen':
			$fp = fopen($mydata['basic_url'],"r") or die('<br/><font color="red">ERROR: ['.mysql_error().']</font> Unable to extract initial data from '.$mydata['basic_url']);
			$filedata = fread($fp,8192);
			fclose($fp);
		break;
	}

	$bloggerId = substr($filedata, strpos($filedata,'<atom:id>')+9, strpos($filedata,'</atom:id>')-strpos($filedata,'<atom:id>')-9);
	$mydata['bloggerId'] = preg_replace('/tag:\w+\.\w+\,\d+:blog-/','',$bloggerId);

	// Too lazy to parse XML
	$mydata['totalPosts'] = substr($filedata, strpos($filedata,'<openSearch:totalResults>')+25, strpos($filedata,'</openSearch:totalResults>')-strpos($filedata,'<openSearch:totalResults>')-25);

	$numLoop = ceil($mydata['totalPosts']/$mydata['numposts']);
	$mydata['numLoop'] = $numLoop;
	$mydata['currLoop'] = $numLoop;
}

function ara_blogger_import_getnumcomments($Post_ID)
{
	global $mydata;
	$comment_basic_url = 'http://'.$mydata['blogger'].'.blogspot.com/feeds/XXXXXXXXXXXX/comments/full?alt=rss&max-results=1&start-index=200000';
	$comment_basic_url = str_replace('XXXXXXXXXXXX',$Post_ID,$comment_basic_url);
	switch($mydata['mechanism'])
	{
		case 'curl':
			$filedata = ara_blogger_import_getCURL($comment_basic_url);
		break;
		case 'fopen':
			$fp = fopen($comment_basic_url,"r") or die('<br/><font color="red">ERROR: ['.mysql_error().']</font> Unable to comment information from '.$mydata['basic_url']);
			$filedata = fread($fp,8192);
			fclose($fp);
		break;
	}

	// Too lazy to parse XML
	$numComments = substr($filedata, strpos($filedata,'<openSearch:totalResults>')+25, strpos($filedata,'</openSearch:totalResults>')-strpos($filedata,'<openSearch:totalResults>')-25);
	return $numComments;
}

function ara_blogger_import_page()
{
	global $mydata, $wpdb, $tablecomments;
	if (isset($_POST['blogger']) && !isset($_POST['next']))
	{
		$mydata['allatonce'] = $_POST['allatonce'];
		if(strlen($mydata['allatonce']) > 0)
			$mydata['numposts'] = 100;
		else
			$mydata['numposts'] = $_POST['numposts'];
		// Get general variables
		$basic_url = 'http://'.$_POST['blogger'].'.blogspot.com/feeds/posts/full?alt=rss&max-results=1&start-index=200000';
		$post_url = 'http://'.$_POST['blogger'].'.blogspot.com/feeds/posts/full?alt=rss&max-results='.$mydata['numposts'].'&start-index=';
		$comment_url = 'http://'.$_POST['blogger'].'.blogspot.com/feeds/XXXXXXXXXXXX/comments/full?alt=rss&max-results=100&start-index=';

		// Decide which file mechanism to use - 1. fopen, 2. curl
		if(ini_get('allow_url_fopen'))
		{
			$mechanism = 'fopen';
		}
		else if(function_exists('curl_init'))
		{
			$mechanism = 'curl';
		}
		else
		{
			$mechanism = 'null';
			echo '<h2>Blogger RSS Import v1.1</h2>';
			echo 'Sorry, fopen is not allowed to open URLs (<a target="_blank" href="http://www.php.net/manual/en/ref.filesystem.php#ini.allow-url-fopen">allow_url_fopen</a>), and no <a href="http://www.php.net/manual/en/ref.curl.php" target="_blank">curl extension</a> is found on your host. Process aborted.';
			exit;
		}

		$mydata['blogger'] = $_POST['blogger'];
		$mydata['basic_url'] = $basic_url;
		$mydata['post_url'] = $post_url;
		$mydata['comment_url'] = $comment_url;
		$mydata['mechanism'] = $mechanism;
		$mydata['importhalo'] = $_POST['importhalo'];
		$mydata['maintainperma'] = $_POST['maintainperma'];
		$mydata['matchcats'] = $_POST['matchcats'];
		$mydata['matchauthors'] = $_POST['matchauthors'];
		$mydata['nocomments'] = $_POST['nocomments'];

		ara_blogger_import_getmetadata($_POST);
		if(isset($_POST['skip']) && strlen($_POST['skip']) > 0)
			ara_blogger_import_process(true);
		else
			ara_blogger_import_process();
	}
	else if (isset($_POST['next']))
	{
		global $mydata;
		// Manipulate mydata first
		$mydata['blogger'] = $_POST['blogger'];
		$mydata['bloggerId'] = $_POST['bloggerId'];
		$mydata['numposts'] = $_POST['numposts'];
		$mydata['totalPosts'] = $_POST['totalPosts'];
		$mydata['numLoop'] = $_POST['numLoop'];
		$mydata['currLoop'] = $_POST['currLoop']-1;
		$mydata['basic_url'] = $_POST['basic_url'];	// this might not be needed anymore
		$mydata['post_url'] = $_POST['post_url'];
		$mydata['comment_url'] = $_POST['comment_url'];
		$mydata['mechanism'] = $_POST['mechanism'];
		$mydata['importhalo'] = $_POST['importhalo'];
		$mydata['maintainperma'] = $_POST['maintainperma'];
		$mydata['matchcats'] = $_POST['matchcats'];
		$mydata['matchauthors'] = $_POST['matchauthors'];
		$mydata['nocomments'] = $_POST['nocomments'];
		ara_blogger_import_process();
	}
	else if (isset($_POST['skip']))
	{
		global $mydata;
		// Manipulate mydata first
		$mydata['blogger'] = $_POST['blogger'];
		$mydata['bloggerId'] = $_POST['bloggerId'];
		$mydata['numposts'] = $_POST['numposts'];
		$mydata['totalPosts'] = $_POST['totalPosts'];
		$mydata['numLoop'] = $_POST['numLoop'];
		$mydata['currLoop'] = $_POST['currLoop']-1;
		$mydata['basic_url'] = $_POST['basic_url'];	// this might not be needed anymore
		$mydata['post_url'] = $_POST['post_url'];
		$mydata['comment_url'] = $_POST['comment_url'];
		$mydata['mechanism'] = $_POST['mechanism'];
		$mydata['importhalo'] = $_POST['importhalo'];
		$mydata['maintainperma'] = $_POST['maintainperma'];
		$mydata['matchcats'] = $_POST['matchcats'];
		$mydata['matchauthors'] = $_POST['matchauthors'];
		$mydata['nocomments'] = $_POST['nocomments'];
		ara_blogger_import_process(true);
	}
	else if(isset($_POST['email0']))
	{
		echo '<div class=wrap>';
		echo '<h2>Blogger RSS Import v1.1</h2>';
		for($x=0;$x<$_POST['commentcount'];$x++)
		{
			if($_POST['email'.$x] != '' || $_POST['website'.$x] != '')
			{
				$sql = "UPDATE $tablecomments SET ";
				if($_POST['email'.$x] != '') $sql.= 'comment_author_email = \''.$_POST['email'.$x].'\' ';
				if($_POST['email'.$x] != '' && $_POST['website'.$x] != '') $sql.=',';
				if($_POST['website'.$x] != '') $sql.= 'comment_author_url=\''.$_POST['website'.$x].'\' ';
				$sql.= 'WHERE comment_author = \''.$_POST['author'.$x].'\'';
				$wpdb->query($sql);
				echo '<br>Updated comment author '.$_POST['author'.$x];
			}
		}
		echo '<p>Now you\'re really done!</p>';
		echo '</div>';
	}
	else
	{
		global $wpdb, $tableusers, $tablecategories;

		?>
		<div class=wrap>
			<h2>Blogger RSS Import v1.1</h2>
			<form method="POST">
				<table border="0">
					<tr><td>Blogger ID:</td><td><input type="text" name="blogger">.blogspot.com</td></tr>
					<tr>
						<td>Default Author:</td>
						<td>
							<select name="user">
								<?php
									$usernames = $wpdb->get_results("SELECT ID, user_login, user_nicename FROM $tableusers");
									foreach ($usernames as $userdat)
									{
										echo '<option value="'.$userdat->ID.'">'.$userdat->user_login.' ('.$userdat->user_nicename.')</option>';
									}
								?>
							</select>
						</td>
					</tr>
					<tr>
						<td colspan="2">
							<input type="checkbox" name="matchauthors" onClick="if(this.checked) this.form.user.disabled = true; else this.form.user.disabled = false;" value="true"> Match authors from my blogger to WordPress, create if not exist
						</td>
					</tr>
					<tr>
						<td>Default Category:</td>
						<td>
							<select name="category">
								<?php
									$categories = $wpdb->get_results("SELECT cat_ID, cat_name FROM $tablecategories");
									foreach ($categories as $catdat)
									{
										echo '<option value="'.$catdat->cat_ID.'">'.$catdat->cat_name.'</option>';
									}
								?>
							</select>
						</td>
					</tr>
					<tr>
						<td colspan="2">
							<input type="checkbox" onClick="if(this.checked) this.form.category.disabled = true; else this.form.category.disabled = false;" name="matchcats" value="true"> Match categories from my blogger to WordPress, create if not exist
						</td>
					</tr>
					<tr>
						<td>Number of posts per import cycle:</td>
						<td>
							<select name="numposts">
								<option value="5">5</option>
								<option value="10">10</option>
								<option value="25">25</option>
								<option value="50">50</option>
								<option value="75">75</option>
								<option value="100" selected>100</option>
							</select>
							*Change this only if you encounter problems due to bandwidth or server speed or you encounter problems with <a target="_blank" href="http://www.php.net/manual/en/ref.info.php#ini.max-execution-time">max_execution_time</a> setting
						</td>
					</tr>
					<tr>
						<td colspan="2">
							<input type="checkbox" name="allatonce" onClick="if(this.checked) this.form.numposts.disabled = true; else this.form.numposts.disabled = false;" value="true"> I want to import everything all at once as my server connection to blogger is very fast and my server has big <a target="_blank" href="http://www.php.net/manual/en/ref.info.php#ini.max-execution-time">max_execution_time</a> setting
						</td>
					</tr>
					<tr>
						<td colspan="2">
							<input type="checkbox" name="importhalo" value="true"> Haloscan friendly import (to be used with <a target="_blank" href="http://justinsomnia.org/2005/06/importing-haloscan-comments-into-wordpress/">this</a>)
						</td>
					</tr>
					<tr>
						<td colspan="2">
							<input type="checkbox" name="nocomments" value="true"> <font color="red">Do not import comments</font> (useful if your blog only uses Haloscan, for example)
						</td>
					</tr>
					<tr>
						<td colspan="2">
							<input type="checkbox" name="maintainperma" value="true"> Maintain my permalinks structure (the post name)
						</td>
					</tr>
					<tr>
						<td colspan="2">
							<input type="submit" value="Import">&nbsp;<input type="submit" name="skip" value="Skip First Cycle &gt;&gt;">
						</td>
					</tr>
				</table>
			</form>
		</div>
<?php
	}
}

function ara_add_blogger_import_page()
{
	if (function_exists('add_management_page'))
	{
		add_management_page('Blogger RSS Import', 'Blogger RSS Import', 10, basename(__FILE__), 'ara_blogger_import_page');
	}
}

add_action('admin_menu', 'ara_add_blogger_import_page');
?>